/*
*    (structures.h)
*/



typedef struct TIMER{
 int state;
 unsigned int timeout;
} TIMER;




typedef struct FIFO8{
  FIFO_STATUS status;
  int         size;
  int         readP;
  int         writeP;
  char        data[512];
} FIFO8;




typedef struct BootInfo{
  char    AsmSkipJMPCode[4];
  address fsAddress;
  address vramAddress;
  unsigned short int   vramXMax;
  unsigned short int   vramYMax;
  unsigned short int   vramLine;
  unsigned short int   padding1;
} BootInfo;




typedef struct VramInfo{
  address              baseAddress;
  unsigned short int   xMax;
  unsigned short int   yMax;
} VramInfo;




typedef struct GlobalDescriptor{
  short    limit1;
  short    base1;
  char     base2;
  char     info;
  char     limit2;
  char     base3;
} GlobalDescriptor;




typedef struct IntDescriptor{
  short    offset1;
  short    cs;
  char     unused;
  char     info;
  short    offset2;
} IntDescriptor;




typedef struct ScreenInfo{
  SCREEN_TYPE    type;
  int            cursorX;
  int            cursorY;
  int            xMax;
  int            yMax;
  unsigned char  fontColor;
  short int     *cursorP;
} ScreenInfo;




typedef struct MemoryEntry{
  MEMORY_STATUS status:1;
} MemoryEntry;




typedef struct MemoryInfo{
  unsigned int capacity;
  MemoryEntry    block[0x10000];
} MemoryInfo;




typedef struct MouseInfo{
  struct FIFO8   fifo;
  unsigned int   phase;
  char           data[3];
  int            x;
  int            y;
  BUTTON_STATUS  leftButton;
  BUTTON_STATUS  rightButton;
} MouseInfo;



typedef struct KeyboardInfo{
  char  CapsLock;
  char  NumLock;
  char  ScrollLock;
  char  alt;
  char  ctrl;
  char  shift;
  char  unused[40];
} KeyboardInfo;




typedef struct TimerInfo{
  unsigned int count;
} TimerInfo;




typedef struct TaskInfo{
  taskid         running;
  taskid         active;
  unsigned int   runningNum;
  unsigned int   sleepingNum;
} TaskInfo;



typedef struct{
 int    edi, esi, ebp, ebx, edx, ecx, eax;
 int    ds, es, fs, gs;
 int    eip;
 int    cs;
 int    ss;
 int    esp;
 int    eflags;
} TASK_REGISTERS;



typedef struct TASK{
  TASK_STATUS     status;
  int             priority;
  unsigned int    cr3;
  address         fileAddr;
  FIFO8           keyboard;
  TASK_REGISTERS  regs;
} TASK;





typedef struct TSS{
  int  backlink;
  int  esp0, ss0;
  int  esp1, ss1;
  int  esp2, ss2;
  int  cr3, eip, eflags;
  int  eax, ecx, edx, ebx, esp, ebp, esi, edi;
  int  es, cs, ss, ds, fs, gs;
  int  ldtr;
  int  io;
} TSS;

