;
;   (assembly.asm)
;

%include "asm.inc"

[bits 32]

global hlt, cli, sti, clihlt, stihlt
global lgdt, lidt, ltr
global initPIC, enablePaging, enableMemoryCache, disableMemoryCache
global in8, out8
global asm_runTask, asm_saveTask
global apmShutdown



hlt:
hlt
ret


cli:
cli
ret


sti:
sti
ret


clihlt:
cli
hlt
ret


stihlt:
sti
hlt
ret

lgdt:
lgdt [GDTR_ADDRESS]
ret


lidt:
lidt [LDTR_ADDRESS]
ret


ltr:
ltr  [esp+4]




initPIC:
mov  al, 0xf8
out  0x21, al

mov  al, 0xef
out  0xa1, al
ret




enablePaging:
mov eax, [esp+4]
mov cr3, eax

mov eax, cr0
or  eax, 0x80000000
mov cr0, eax
ret




enableMemoryCache:
mov  eax, cr0
or   eax, 0x60000000
mov  cr0, eax
ret




disableMemoryCache:
mov  eax, cr0
and  eax, 0x9fffffff
mov  cr0, eax
ret




in8:
mov  eax, 0
mov  dx,  [esp+4]
in   al,  dx
ret


out8:
mov  dx, [esp+4]
mov  al, [esp+8]

out  dx, al
ret




esp_tmp dd 0

asm_runTask:

mov  eax,      [esp+8]
mov  cr3,      eax


mov  [TSS_ADDRESS+4], esp
mov  esp,      [esp+4]
mov  [esp_tmp], esp

pop  edi
pop  esi
pop  ebp
pop  ebx
pop  edx
pop  ecx
pop  eax

mov   esp,  [TSS_ADDRESS+4]
push ebp
mov   ebp, [esp_tmp]


push dword [ebp+52]  ;SS
push dword [ebp+56]  ;ESP
push dword [ebp+60]  ;EFLAGS
push dword [ebp+48]  ;CS
push dword [ebp+44]  ;EIP

push ax
mov  ax, GDT_USERDATA_SELECTOR+3
mov  ds, ax
mov  es, ax
mov  fs, ax
mov  gs, ax
pop  ax

add  esp, 16
pop  ebp
sub  esp, 20

iretd



asm_saveTask:
pushad

mov edi, [esp+36]

mov ebp, esp
add ebp, 0xb4

mov eax, [ebp+0]
mov [edi+44], eax  ;EIP
mov eax, [ebp+4]
mov [edi+48], eax  ;CS
mov eax, [ebp+8]
mov [edi+60], eax  ;EFLAGS
mov eax, [ebp+12]
mov [edi+56], eax  ;ESP
mov eax, [ebp+16]
mov [edi+52], eax  ;SS

mov ebp, esp
add ebp, 0x94

mov eax, [ebp+0]
mov [edi+0], eax  ;EDI
mov eax, [ebp+4]
mov [edi+0], eax  ;ESI
mov eax, [ebp+8]
mov [edi+0], eax  ;EBP
mov eax, [ebp+12]
mov [edi+0], eax  ;EBX
mov eax, [ebp+16]
mov [edi+0], eax  ;EDX
mov eax, [ebp+20]
mov [edi+0], eax  ;ECX
mov eax, [ebp+24]
mov [edi+0], eax  ;EAX

mov eax, [ebp+0]
mov [edi+0], eax  ;DS
mov eax, [ebp+0]
mov [edi+0], eax  ;ES
mov eax, [ebp+0]
mov [edi+0], eax  ;FS
mov eax, [ebp+0]
mov [edi+0], eax  ;GS



popad
ret






apmShutdown:
cli

lidt  [idtr2]
lgdt  [gdtr2]


mov  eax, cr0
and  eax, 0x7fffffff  ;ページングを無効化
mov  cr0, eax

mov  eax, 0
mov  cr3, eax ;TLBをフラッシュ

; APMでのシャットダウンするコード (boot2.asm) へjmpする
jmp  dword 0x18:0x28


idtr2:
dw 0x7ff
dd 0

gdtr2:
dw 0x28
dd 0x8200

