;
;  (boot2.asm)
;

%include "asm.inc"

[bits 16]
[org 0]


mov     ax, 0x800
mov     ds, ax
mov     ax, 0
mov     ss, ax


;ビデオモード指定
videoMode:
mov  si, msgSelectVideoMode
call print
mov  ah, 0x00
int  0x16

cmp  al, '1'
je   .text

cmp  al, '2'
je  .graphic

jmp videoMode


;
; GUI モード
;
.graphic:
mov     ax, 0
mov     es, ax
mov     di, 0xd000

; VESAの情報を取得
mov     ax, 0x4f00
int     0x10

cmp     al, 0x4f
jne     .err
cmp     ah, 0
jne     .err


; 画面モードの情報を取得
mov     di, 0xd200
mov     ax, 0x4f01
mov     cx, GUI_VBE_MODE
int     0x10

cmp     al, 0x4f
jne     .err
cmp     ah, 0
jne     .err


mov     eax, [es:0xd200+0x28]
mov  dword   [es:BOOTINFO_ADDRESS+BINFO_VRAM_ADDRESS], eax
mov     ax, [es:0xd200+0x12]
mov  word    [es:BOOTINFO_ADDRESS+BINFO_VRAM_X_MAX],   ax
mov     ax, [es:0xd200+0x14]
mov  word    [es:BOOTINFO_ADDRESS+BINFO_VRAM_Y_MAX],   ax
mov     ax, [es:0xd200+0x10]
mov  word    [es:BOOTINFO_ADDRESS+BINFO_VRAM_LINE],    ax


; 画面モードを設定
mov     ax, 0x4f02
mov     bx, 0x4000+GUI_VBE_MODE
int     0x10

cmp     al, 0x4f
jne     .err
cmp     ah, 0
jne     .err

jmp     apm


.err:
mov  si, msgVramIsNotSupported
call print
mov  si, msgSystemHalted
call print

.hlt:
hlt
jmp .hlt



;
; CUI モード
;
.text:
mov     ah, 0x00
mov     al, 0x03
int     0x10
jmp     apm





;APM BIOS
apm:
mov ax, 0x5300
mov bx, 0
int 0x15
jc  .apm_next

mov  ax, 0x5301
mov  bx, 0
int  0x15
.apm_next:



;PIC初期化
mov  al, 0xff
out  0xa1, al
nop
out  0x21, al
cli

mov  al, 0x11
out  0x20, al
nop
out  0xa0, al

mov  al, 0x20
out  0x21, al
mov  al, 0x28
out  0xa1, al

mov  al, 0x04
out  0x21, al
mov  al, 0x02
out  0xa1, al

mov  al, 0x01
out  0x21, al
nop
out  0xa1, al



;A20有効化
call waitKeyboard
mov  al,   0xd1
out  0x64, al
call waitKeyboard
mov  al,   0xdf
out  0x60, al
jmp loadGDT


waitKeyboard:
in      al, 0x64
and     al, 0x02
in      al, 0x60
jnz     waitKeyboard
ret



loadGDT:
lgdt    [gdtr]

mov     eax, cr0
or      eax, 0x00000001  ;プロテクトモードにして
and     eax, 0x7fffffff  ;ページングを無効化
mov     cr0, eax

jmp dword 24:clear+0x8000


;プロテクトモード
[bits 32]

clear:
mov     ax, 16
mov     ds, ax
mov     es, ax
mov     fs, ax
mov     gs, ax
mov     ss, ax
mov     esp, 0x9fffff

;kernel.bin読み込み
cld
mov     ecx, KERNEL_BIN_LOAD_SIZE
mov     esi, 0x8400
mov     edi, 0x100000
rep     movsb


jmp dword 8:0  ;boot3へ移行


[bits 16]

print:
push ax
push bx
mov  ah, 0x0e
mov  bh, 0
mov  bl, 15

.loop:
mov  al, [si]
cmp  al, 0
je   .ret
int  0x10
inc  si
jmp  .loop

.ret:
pop  bx
pop  ax
ret

msgSelectVideoMode      db  0x0d,0x0a,"Select Video Mode (1: text) (2: graphic): ",0
msgVramIsNotSupported   db  0x0d,0x0a,"(!) VRAM is not supported.",0
msgSystemHalted         db  0x0d,0x0a,"System halted.",0

align 8

gdtr:
dw gdtEnd - gdtStart -1
dd gdtStart + 0x8000

align 8

gdtStart:
;NULL Descriptor
dw 0
dw 0
db 0
db 0
db 0
db 0

;Code Descriptor (for boot3)
dw 0xffff
dw 0
db 0x10
db 0x9b
db 0xcf
db 0

;Data Descriptor
dw 0xffff
dw 0
db 0
db 0x93
db 0xcf
db 0

;Code Descriptor (for boot2)
dw 0xffff
dw 0
db 0
db 0x9b
db 0xcf
db 0

gdtEnd:


times   512-($-$$) db 0

gdt2Start:
;NULL Descriptor
dw 0
dw 0
db 0
db 0
db 0
db 0

;Code Descriptor (for boot3)
dw 0xffff
dw 0
db 0x10
db 0x9b
db 0xcf
db 0

;Data Descriptor
dw 0xffff
dw 0
db 0
db 0x93
db 0xcf
db 0

;Code Descriptor 16(for APM)
dw 0xffff
dw 0x8200
db 0
db 0x9a
db 0
db 0

;Data Descriptor 16(for APM)
dw 0xffff
dw 0x8200
db 0
db 0x92
db 0
db 0




;APM Shutdown
[bits 16]

mov eax, cr0
and eax, 0x7ffffffe
mov cr0, eax

jmp dword 0:0x8300

times   768-($-$$) db 0

mov ax, 0
mov ss, ax
mov esp, 0x8200

mov ax, 0x530e
mov bx, 0
mov cx, 0x0101
int 0x15
jc err2

mov ax, 0x530f
mov bx, 1
mov cx, 1
int 0x15
jc err2

mov ax, 0x5308
mov bx, 1
mov cx, 1
int 0x15
jc err2

mov ax, 0x5307
mov bx, 1
mov cx, 3
int 0x15
jc err2

err2:
hlt
jmp $


times   1024-($-$$) db 0

