/*
*    (boot3.c)
*/

#include "kernel.h"

BootInfo       *bootInfo;
VramInfo        vram;
TimerInfo       timer;
KeyboardInfo    keyboard;
MouseInfo       mouse;
ScreenInfo      screen;
MemoryInfo      memory;
TaskInfo        task;

TASK            Tasks[TASK_MAX];
TIMER           Timer[TIMER_MAX];




/*
*  Function: boot3
*
*    各初期化関数を呼び出す
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void boot3 (void){

  cli();

  bootInfo = (BootInfo *) BOOTINFO_ADDRESS;
  vram.baseAddress = bootInfo->vramAddress;
  vram.xMax        = bootInfo->vramXMax;
  vram.yMax        = bootInfo->vramYMax;
  screen.xMax      = SCREEN_X_MAX;
  screen.yMax      = SCREEN_Y_MAX;
  screen.fontColor = SCREEN_FONT_COLOR;


  if(vram.baseAddress != 0x00000000)
    screen.type      = SCREEN_GUI;
  else
    screen.type      = SCREEN_CUI;


  initScreen();
  initDescriptors();
  initPIC();
  initTimer();
  initKeyboard();
  initMemory();
  initTask();
  initFS();
  initMouse();

  simpleShell();
}

