/*
*   (fifo.c)
*/

#include "kernel.h"



/*
*  Function: initFifo8
*
*    FIFO8型のFIFOバッファを初期化する
*
*  Parameters:
*
*    fifo -  FIFOバッファへのポインタ
*
*  Returns:
*
*    なし
*
*/

void initFifo8 (FIFO8 *fifo){


  fifo->readP   = 0;
  fifo->writeP  = 0;
  fifo->size    = FIFO8_SIZE;
  fifo->status  = FIFO_EMPTY;

  return;
}




/*
*  Function: putFifo8
*
*    FIFO8型のFIFOバッファにデータを追加する
*
*  Parameters:
*
*    fifo -  FIFOバッファへのポインタ
*    data -  追加するデータ
*
*  Returns:
*
*    なし
*
*/

void putFifo8 (struct FIFO8 *fifo, char data){

  cli();

  fifo->status             = FIFO_EXIST;
  fifo->data[fifo->writeP] = data;
  fifo->writeP++;

  if(fifo->writeP == fifo->size)
    fifo->writeP = 0;

  sti();

  return;
}




/*
*  Function: getFifo8
*
*    FIFO8型のFIFOバッファよりデータを取得する
*
*  Parameters:
*
*    fifo -  FIFOバッファへのポインタ
*
*  Returns:
*
*    バッファにデータが残っている場合にはそのデータひとつ、残っていない場合には-1を返します。
*
*/

int getFifo8 (struct FIFO8 *fifo){

 char data;

  cli();

  if(fifo->readP == fifo->writeP){
    fifo->status = FIFO_EMPTY;
    return -1;
  }

  data = fifo->data[fifo->readP];
  fifo->readP++;


  if(fifo->readP == fifo->size)
    fifo->readP = 0;

  sti();

  return  data;
}




/*
*  Function: getFifoStatus8
*
*    FIFO8型のFIFOバッファの状態を取得する
*
*  Parameters:
*
*    fifo -  FIFOバッファへのポインタ
*
*  Returns:
*
*    バッファの状態を返します。
*
*/

FIFO_STATUS getFifo8Status (struct FIFO8 *fifo){

  return fifo->status;
}

