;
;Resea IPL
;

%include "asm.inc"

[bits    16]
[org     0x7c00]


jmp entrypoint


;ResFS
            db 0  ;パディング
fsAddress   dd 0  ;Resfsが始まるアドレス
vramAddress dd 0  ;VRAMのアドレス
vramXMax    dw 0  ;VRAMの横の最大値
vramYMax    dw 0  ;VRAMの縦の最大値
vramLine    dw 0
            dw 0
align       16



entrypoint:

;レジスタ初期化
mov     ax, cs
mov     ss, ax
mov     ds, ax
mov     sp, 0x7c00
mov     ax, 0xb800
mov     gs, ax
mov     di, 0

mov     ah, 0x00
mov     al, 0x03
int     0x10

mov  si, msgLoading
call print

call load ;カーネル読み込み

mov  si, msgOK
call print

jmp dword  0:0x08000  ;boot2へ移行



load:
jmp read13e

load_ret:
ret


;拡張INT 0x13を使った読み込み (ハードディスクおよびUSBメモリ用)
read13e:
mov  si, msgUsingINT13e
call print

;拡張INT 0x13が使えるか確認

mov ah, 0x41
mov bx, 0x55aa
;mov  dl, 0 ;driveはBIOSから渡されたのを使う
int 0x13
jc  read13
cmp bx, 0xaa55
jne read13
and cl, 1
cmp cl, 1
jne read13

mov ah, 0x42
mov al, 0
;mov  dl, 0 ;driveはBIOSから渡されたのを使う
mov si, DAP
int 0x13
jc  read13

jmp load_ret


;INT 0x13を使った読み込み (フロッピー用)
mov  si, msgUsingINT13
call print

read13:
mov  ax, BOOT2_ADDRESS
mov  es, ax
;mov  dl, 0 ;driveはBIOSから渡されたのを使う

mov  dh, 0  ;head
mov  ch, 0  ;cylinder
mov  cl, 2  ;sector


.read:

mov  ah, 2
mov  al, 1
mov  bx, 0
mov  dl, 0

int 0x13
jc  error

mov ax, es
add ax, 0x20
mov es, ax

inc  cl
cmp  cl, CYLINDERS
jbe  .read

mov  cl, 1
inc  dh
cmp  dh, HEADS
jbe  .read

mov  si, msgProgressSymbol
call print

mov  dh, 0
inc  ch
cmp  ch, LOAD_SIZE
jbe  .read

jmp load_ret



error:
mov  si, msgLoadingError
call print

.hlt:
hlt
jmp  .hlt

print:
push ax
push bx
mov  ah, 0x0e
mov  bh, 0
mov  bl, 15

.loop:
mov  al, [si]
cmp  al, 0
je   .ret
int  0x10
inc  si
jmp  .loop

.ret:
pop  bx
pop  ax
ret



;DAP:
db 0x10
db 0
dw 10
dw 0
dw BOOT2_ADDRESS
dq 0

DAP:
db	0x10		; パケットサイズ(=16)
db	0x00		; 予約
db	90		    ; Blocks = 31.5Kbyte
db	0			; 予約
dw	0x8000		; 読み込みオフセット位置
dw	0x0000		; 読み込みセグメント位置
dd	1			; 読み込み先頭セクタ番号
dd	0			; 読み込み先頭セクタ番号上位32ビット



msgUsingINT13       db "Using INT 0x13.",            0x0d,0x0a,0
msgUsingINT13e      db "Using extended INT 0x13",    0x0d,0x0a,0
msgLoading          db "Loading ",                   0x0d,0x0a,0
msgLoadingError     db 0x0d, 0x0a, "Loading error.", 0
msgProgressSymbol   db ".",                          0
msgOK               db "ok",                         0x0d,0x0a,0



times   446-($-$$) db 0


;パーティションテーブル
dd 0, 0, 0, 0
dd 0, 0, 0, 0
dd 0, 0, 0, 0
dd 0, 0, 0, 0


;ブートシグネチャ
dw 0xaa55

