/*
 *   (mouse.c)
 */

#include "kernel.h"



/*
*  Function: moveMouseCursor
*
*    マウスカーソルを移動する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void moveMouseCursor (void){

  cli();


  if((mouse.data[0] & 0x10) != 0)
    mouse.x -= abs(mouse.data[1]);
  else
    mouse.x += mouse.data[1];

  if((mouse.data[0] & 0x20) != 0)
    mouse.y += abs(mouse.data[2]);
  else
    mouse.y -= mouse.data[2];



  if(mouse.x < 0)
    mouse.x  = 0;

  if(mouse.y < 0)
    mouse.y  = 0;

  if(mouse.x > vram.xMax)
    mouse.x  = vram.xMax;

  if(mouse.y > vram.yMax)
    mouse.y  = vram.yMax;


  if((mouse.data[0] & 0x01) == 0x01)
    mouse.leftButton = BUTTON_DOWN;
  else
    mouse.leftButton = BUTTON_UP;

  if((mouse.data[0] & 0x02) == 0x02)
    mouse.rightButton = BUTTON_DOWN;
  else
    mouse.rightButton = BUTTON_UP;


  return;
}




/*
*  Function: waitKBCForMouse
*
*    キーボードコントローラを待つ
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void waitKBCForMouse (void){

  while(in8(IOPORT_KBC2)&0x02);
  return;
}




/*
*  Function: initMouse
*
*    マウスを初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool initMouse (void){

  initFifo8(&mouse.fifo);

  waitKBCForMouse();
  out8(IOPORT_KBC2, 0x60);
  waitKBCForMouse();
  out8(IOPORT_KBC1, 0x47);
  waitKBCForMouse();
  out8(IOPORT_KBC2, 0xd4);
  waitKBCForMouse();
  out8(IOPORT_KBC1, 0xf4);

  return true;
}




/*
*  Function: initMouseHandler
*
*    マウス割り込みのハンドラ
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void intMouseHandler (void){

  while(!(in8(IOPORT_KBC2)&0x01));
  putFifo8(&mouse.fifo, in8(IOPORT_KBC1));

  return;
}

