/*
 *   (print.c)
 */

#include "kernel.h"



/*
*  Function: moveCursor
*
*    カーソルを移動する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void moveCursor (void){

  unsigned short cursor = screen.cursorY*screen.xMax+screen.cursorX;

  out8(0x03d4, 15);
  out8(0x03d5, cursor);
  out8(0x03d4, 14);
  out8(0x03d5, cursor>>8);

 return;
}




/*
*  Function: printc
*
*    指定された色で一文字表示する
*
*  Parameters:
*
*    c     -  文字
*    color -  色
*
*  Returns:
*
*    なし
*
*/

void printc (const char c , unsigned char color){

 *screen.cursorP = (screen.fontColor<<8)+0x20;

  if(c == '\t'){

    for(int i=0; i < TAB_SIZE; i++)
      printc(' ', color);

    moveCursor();
    return;
  }

  if(c == '\b'){
    if(screen.cursorX < 1){
      moveCursor();
      return;
    }

     screen.cursorX--;
    *((short int *) CUI_VRAM_ADDRESS+(screen.cursorY*screen.xMax+screen.cursorX)) = (screen.fontColor<<8)+0x20;

     moveCursor();
    return;
  }


  if(c == '\n' || screen.cursorX == screen.xMax){
     screen.cursorX = 0;
     screen.cursorY++;

     if(screen.cursorY == screen.yMax){
       for(int i = 0; i < screen.yMax; i++)
         for(int j = 0; j < screen.xMax; j++){
           *((char *) CUI_VRAM_ADDRESS+(i*screen.xMax+j)*2)   = *((char *) CUI_VRAM_ADDRESS+((i+1)*screen.xMax+j)*2);
           *((char *) CUI_VRAM_ADDRESS+(i*screen.xMax+j)*2+1) = *((char *) CUI_VRAM_ADDRESS+((i+1)*screen.xMax+j)*2+1);
           screen.cursorX = 0; screen.cursorY = screen.yMax-1;
         }
       }

      for(int i = 0; i < screen.xMax; i++)
        *((short int *) CUI_VRAM_ADDRESS+(screen.cursorY*screen.xMax+screen.cursorX)+i) = (screen.fontColor<<8)+0x20;

  }else{

    *((char *) CUI_VRAM_ADDRESS+(screen.cursorY*screen.xMax+screen.cursorX)*2)   = c;
    *((char *) CUI_VRAM_ADDRESS+(screen.cursorY*screen.xMax+screen.cursorX)*2+1) = color;

    screen.cursorP =  (short int *) CUI_VRAM_ADDRESS+(screen.cursorY*screen.xMax+screen.cursorX)+1;
    screen.cursorX++;
  }


  moveCursor();
  return;
}




/*
*  Function: printchar
*
*    一文字表示する
*
*  Parameters:
*
*    c     -  文字
*
*  Returns:
*
*    なし
*
*/

void printchar (const char c){

  printc(c, screen.fontColor);
  return;
}




/*
*  Function: print
*
*    書式文字列に従って文字列を表示する
*
*  Parameters:
*
*    c     -  文字
*
*  Returns:
*
*    なし
*
*/

void print (const char *format, ...){

 va_list vargs;

  va_start(vargs, format); 
  vprintf(format, vargs);
  va_end(vargs); 

  return;
}

