/*
*    (test.c)
*/


#include "kernel.h"


/*
*  about: このファイルについて (test.c)
*
*    このファイル内の関数は、動作テスト用です。
*
*/

void mouseTest (void);
void elfTest   (void);


#define  SHELL_PROMPT  "> " 

void simpleShell (void){

 int c;
 char str[64];
 taskid tid;


  //TEST: マウステスト
  if(vram.baseAddress != 0)
    mouseTest();


  tid         = 0xe;
  task.active = tid;

  Tasks[tid].keyboard.readP  = 0;
  Tasks[tid].keyboard.writeP = 0;
  Tasks[tid].keyboard.size   = FIFO8_SIZE;

  for(;;){
    sti();
    print(SHELL_PROMPT);

    str[0] = '\0';

    for(int i=0 ;; i++){

      if((c = fgetc(tid)) == EOF){
        i--;
        continue;
      }

      if(c == '\n'){
        str[i] = '\0';
        break;
      }


      if(c == '\b'){
        str[i] = '\0';

        if(i > 0)
        i-=2;

      }else{
        str[i] = c;
      }

      printchar(c);

      if(strlen(SHELL_PROMPT) > screen.cursorX)
        break;

      hlt();
    }

    print("\n");

    //改行だけの場合
    if(str[0] == '\0')
      continue;


    //画面のクリア
    if(strcmp(str, "clear")    == 0){
      clearScreen();
      continue;
    }

    //シャットダウン
    if(strcmp(str, "shutdown") == 0)
      apmShutdown();


    //TEST: test*.elfの実行テスト
    if(strcmp(str, "elf") == 0){
      elfTest();
      continue;
    }


    print("unknown command: %s\n", str);
  }
}




bool flag=1;

void elftestHd (void){

  print("\x1b[33m%c\x1b[39m", *((char*) 0x100));

  if(task.running == 0xa && flag==1){
    flag=2;
    startTask(0x53000-0x03);
  }

  if(flag == 2){
    startTask(0x55000-0x03);
    flag = 10;
  }

  return;
}




void elfTest(void){

 bool res;
 unsigned char *file;

  print("\n ===== ELF TEST =====\n\n");
  print("\n");


  file =  (void *) 0x50000;

  print("load \"%s\"... ", "test1.elf");
  res = fileLoad((void *)file,  "test1.elf");
  print((res)?"\x1b[42mok\x1b[49m\n":"\x1b[41merr!\x1b[49m");

  file =  (void *) 0x52000;

  print("load \"%s\"... ","test2.elf");
  res = fileLoad((void *)file, "test2.elf");
  print((res)?"\x1b[42mok\x1b[49m\n":"\x1b[41merr!\x1b[49m");

  file =  (void *) 0x54000;

  print("load \"%s\"... ","test3.elf");
  res = fileLoad((void *)file, "test3.elf");
  print((res)?"\x1b[42mok\x1b[49m\n":"\x1b[41merr!\x1b[49m");


  startTask(0x51000-0x03);
  switchTask();


 return;
}



static char mouseCursor[]={
  '_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_',
  '_','_','_','_','#','_','_','_','_','_','_','_','_','_','_','_',
  '_','_','_','_','#','#','_','_','_','_','_','_','_','_','_','_',
  '_','_','_','_','#','#','#','_','_','_','_','_','_','_','_','_',
  '_','_','_','_','#','#','#','#','_','_','_','_','_','_','_','_',
  '_','_','_','_','#','#','#','#','#','_','_','_','_','_','_','_',
  '_','_','_','_','#','#','#','#','#','#','_','_','_','_','_','_',
  '_','_','_','_','#','#','#','#','#','#','#','_','_','_','_','_',
  '_','_','_','_','#','#','#','#','#','#','#','#','_','_','_','_',
  '_','_','_','_','#','#','#','#','#','#','#','_','_','_','_','_',
  '_','_','_','_','#','#','#','#','#','_','_','_','_','_','_','_',
  '_','_','_','_','#','#','_','#','#','_','_','_','_','_','_','_',
  '_','_','_','_','#','_','_','_','#','#','_','_','_','_','_','_',
  '_','_','_','_','_','_','_','_','#','#','_','_','_','_','_','_',
  '_','_','_','_','_','_','_','_','_','#','#','_','_','_','_','_',
  '_','_','_','_','_','_','_','_','_','#','#','_','_','_','_','_',
};


void mouseTest(){

 mouse.phase = 0;
 mouse.x     = 0;
 mouse.y     = 0;
 int data2;
 unsigned char data;

  for(unsigned int y=0; y < vram.yMax; y++)
    for(unsigned int x=0; x < vram.xMax; x++)
      paint(x, y, 215);


  for(;;){

    sti();
    while(getFifo8Status(&mouse.fifo) == FIFO_EMPTY)hlt();
    cli();

    data2 = getFifo8(&mouse.fifo);
    if(data2 == -1)
      data2 = 0;

    data = (char) data2;



    if(mouse.phase == 0){

      if(data == 0xfa)
        mouse.phase = 1;

    }else if(mouse.phase == 1){

      mouse.data[0] = data;
      mouse.phase   = 2;

      if((data & 0x08) == 0x08){
        mouse.data[0] = data;
        mouse.phase   = 2;
      }else{
        mouse.phase = 1;
      }


    }else if(mouse.phase == 2){
      mouse.data[1] = data;
      mouse.phase   = 3;

    }else if(mouse.phase == 3){

      mouse.data[2] = data;
      mouse.phase   = 1;

     for(int y=mouse.y; y < 16+mouse.y; y++)
        for(int x=mouse.x; x < 16+mouse.x; x++)
            paint(x, y, 215);

      moveMouseCursor();

     for(int y=mouse.y; y < 16+mouse.y; y++)
        for(int x=mouse.x; x < 16+mouse.x; x++)
            paint(x, y, (mouseCursor[(x-mouse.x)+((y-mouse.y)*16)]=='#')?0:215);


    }  //if(mouse.phase == 3)
  }  //for(;;)
}  //void mouseTest (void)


