/*
*    (timer.c)
*/

#include "kernel.h"


/*
*  Function: setTimer
*
*    タイマをセットする
*
*  Parameters:
*
*    timerID -  タイマのID
*    timeout -  タイムアウトさせるまでのミリ秒数
*
*  Returns:
*
*    なし
*
*/

void setTimer (unsigned int timerID , unsigned int timeout){

  Timer[timerID].state    = 1;
  Timer[timerID].timeout  = timer.count+timeout;
  return;
}




/*
*  Function: intTimerHandler
*
*    タイマ割り込みのハンドラ
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    なし
*
*/

void intTimerHandler (void){

  timer.count++;

  if(timer.count == TIMER_RESET_COUNT){
    timer.count -= TIMER_RESET_COUNT;

    for(int i = 0; i < TIMER_MAX; i++)
      if(Timer[i].state == 1)
        Timer[i].timeout -= TIMER_RESET_COUNT;
  }

  for(int i = 0; i < TIMER_MAX; i++){
     if(Timer[i].state == 1 && Timer[i].timeout == timer.count)
       Timer[i].state = 2;
  }

  if(Timer[TIMER_TASK].timeout >= timer.count)
    switchTask();
  else
    setTimer(TIMER_TASK, 1);

  return;
}




/*
*  Function: initTimer
*
*    タイマの初期化する
*
*  Parameters:
*
*    なし
*
*  Returns:
*
*    成功した場合はtrue、失敗した場合はfalseを返します。
*
*/

bool initTimer (void){

  out8(IOPORT_TIMER1 , 0x34);
  out8(IOPORT_TIMER2 , 0x9c);
  out8(IOPORT_TIMER2 , 0x2e);

  return true;
}

