#!/usr/bin/env perl
#
#
#  ReseaFSの作成
#
#

use strict;
use warnings;

use File::Basename;
use Perl6::Slurp;

#
#  $ARGV[0]:  FSイメージファイル名
#  $ARGV[1-]: ファイルパス
#


my $flagUsing    = 1;
my $FILES_MAX    = 128;
my $VERSION      = 1;
my $SIGNATURE_1  = 0xa0;
my $SIGNATURE_2  = "ResFS";
my $COMPATIBLE   = 0xcc;
my $INCOMPATIBLE = 0xff;
my $ST_INFO_SIZE = 15;
my $ST_FILE_SIZE = 44;



# 引数がきちんとあるかチェック
if(@ARGV < 2){
  die "too few arguments.";
}

my $FSFilePath = shift;


my @filelist;
my $i=0;
my $FSFileSize=0;



#ファイルの情報をとっておく
foreach my $filepath (@ARGV){

  $filelist[$i]{"path"} = $filepath;
  $filelist[$i]{"name"} = basename($filepath);
  $filelist[$i]{"size"} = -s $filepath;
  $filelist[$i]{"addr"} = $FSFileSize;

  $FSFileSize += $filelist[$i]{"size"};
  $i++;
}




#
#書き込む
#

open(FH, ">$FSFilePath") or die "File Open Error ($FSFilePath).";
binmode(FH);

#情報
my $tmp;

print FH pack("CA5CCI", $SIGNATURE_1, $SIGNATURE_2, $VERSION, $INCOMPATIBLE, $ST_INFO_SIZE+($FILES_MAX*$ST_FILE_SIZE));

#ファイルのデータベース
for(my $i=0; $i < $FILES_MAX; $i++){

  if(defined($filelist[$i]{"path"})){
    print FH pack("Ia32II", $flagUsing, $filelist[$i]{"name"}, $filelist[$i]{"addr"}, $filelist[$i]{"size"});
  }else{
    print FH pack("Ia32II");
  }
}


#ファイル本体
for(my $i=0; defined($filelist[$i]{"name"}); $i++){

  print FH slurp($filelist[$i]{"path"});
}

close(FH);
exit(0);

